/*  This file is part of FediBlock-backend.
 *  Copyright © 2020 tastytea <tastytea@tastytea.de>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, version 3.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FEDIBLOCK_BACKEND_GIT_HPP
#define FEDIBLOCK_BACKEND_GIT_HPP

#include "cgi.hpp"

#include <git2.h>

#include <cstdint>

namespace FediBlock::git
{

using std::uint64_t;

// Throws runtime_error on error.
void check(int error);

// Acquire credentials for SSH.
int cred_acquire(git_cred **cred, const char *url,
                 const char *username_from_url, unsigned int allowed_types,
                 void *payload);

// Clone the git repo.
void clone();

// FIXME: This will collapse when branches are deleted, use pull request IDs?
// Get highest branch-ID in use.
[[nodiscard]] uint64_t get_last_id();

// Make a new branch for preparing the pull request.
void create_branch();

// Make a new commit.
void commit(const cgi::entry_type &entry);

// Push the new commit.
void push();

} // namespace FediBlock::git

#endif // FEDIBLOCK_BACKEND_GIT_HPP
